<?php

namespace App\Controller;

use App\Entity\Snip;
use App\Form\ConfirmationType;
use App\Form\SnipType;
use App\Repository\SnipRepository;
use App\Security\Voter\SnipVoter;
use App\Service\SnipParser\Pipeline;
use App\Service\SnipServiceFactory;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

#[Route('/snip', name: 'snip')]
class SnipController extends AbstractController
{
    public function __construct(
        private readonly SnipRepository     $repository,
        private readonly SnipServiceFactory $snipServiceFactory,
    )
    {
    }

    #[Route('/', name: '_index')]
    public function index(): Response
    {
        return $this->render('snip/index.html.twig', [
            'snips' => $this->repository->findByUser($this->getUser()),
            'title' => 'My Snips',
        ]);
    }

    #[Route('/public', name: '_public')]
    public function public(): Response
    {
        return $this->render('snip/index.html.twig', [
            'snips' => $this->repository->findPublic($this->getUser()),
            'title' => 'Public Snips',
        ]);
    }

    #[Route('/single/{snip}', name: '_single')]
    public function single(Snip $snip, Pipeline $pl): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::VIEW, $snip);

        $snipService = $this->snipServiceFactory->createGit($snip);
        return $this->render('snip/single.html.twig', [
            'snip' => $snip,
            'content' => $pl->parse($snipService->get()),
            'branch' => $snipService->getCommit(),
        ]);
    }

    #[Route('/raw/{snip}', name: '_raw')]
    public function raw(Snip $snip, Pipeline $pl, Request $request): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::VIEW, $snip);

        $response = new Response(
            $pl->clean($this->snipServiceFactory->createGit($snip)->get()),
            Response::HTTP_OK,
            ['Content-Type' => 'text/html']
        );
        $response
            ->setVary(['Accept', 'Accept-Encoding'])
            ->setEtag(md5($response->getContent()))
            ->setTtl(3600)
            ->setClientTtl(300);

        if (!$request->isNoCache()) {
            $response->isNotModified($request);
        }

        return $response;
    }

    #[Route('/edit/{snip}', name: '_edit')]
    public function edit(Snip $snip, Request $request): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $form = $this->createForm(SnipType::class, $snip);
        $form->add('Save', SubmitType::class);
        if ($snip->getId()) {
            $form->get('content')->setData($this->snipServiceFactory->createGit($snip)->get());
        }

        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->repository->save($snip);
            $this->snipServiceFactory->createGit($snip)->update($form->get('content')->getData());

            $this->addFlash('success', sprintf('Snip "%s" saved', $snip));

            return $this->redirectToRoute('snip_single', [
                'snip' => $snip->getId(),
            ]);
        }

        return $this->render('snip/edit.html.twig', [
            'snip' => $snip,
            'form' => $form->createView(),
        ]);
    }

    #[Route('/new', name: '_new')]
    public function new(Request $request): Response
    {
        $snip = new Snip();
        $snip->setCreatedAtTodayNoSeconds()
            ->setCreatedBy($this->getUser());

        return $this->edit($snip, $request);
    }

    #[Route('/delete/{snip}', name: '_delete')]
    public function delete(Snip $snip, Request $request): Response
    {
        $this->denyAccessUnlessGranted(SnipVoter::EDIT, $snip);

        $form = $this->createForm(ConfirmationType::class);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->snipServiceFactory->createGit($snip)->delete();
            $this->repository->remove($snip);
            $this->addFlash('success', sprintf('Snip "%s" deleted', $snip));
            return $this->redirectToRoute('snip_index');
        }

        return $this->render('form.html.twig', [
            'message' => sprintf('Do you really want to delete "%s"?', $snip),
            'form' => $form->createView(),
        ]);
    }
}